//
// Copyright 2016 Pixar
//
// Licensed under the Apache License, Version 2.0 (the "Apache License")
// with the following modification; you may not use this file except in
// compliance with the Apache License and the following modification to it:
// Section 6. Trademarks. is deleted and replaced with:
//
// 6. Trademarks. This License does not grant permission to use the trade
//    names, trademarks, service marks, or product names of the Licensor
//    and its affiliates, except as required to comply with Section 4(c) of
//    the License and to reproduce the content of the NOTICE file.
//
// You may obtain a copy of the Apache License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the Apache License with the above modification is
// distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied. See the Apache License for the specific
// language governing permissions and limitations under the Apache License.
//

#include "pxr/pxr.h"

#include "pxr/base/tf/pyObjectFinder.h"
#include "pxr/base/tf/pyUtils.h"
#include "pxr/base/tf/staticData.h"
#include "pxr/base/tf/typeInfoMap.h"

using std::type_info;

using namespace boost::python;

PXR_NAMESPACE_OPEN_SCOPE

static TfStaticData<TfTypeInfoMap<Tf_PyObjectFinderBase const *> > _finders;


void Tf_RegisterPythonObjectFinderInternal(std::type_info const &type,
                                           Tf_PyObjectFinderBase const *finder) {
    _finders->Set(type, finder);
}

object Tf_FindPythonObject(void const *objPtr, std::type_info const &type) {
    Tf_PyObjectFinderBase const *finder = 0;
    if (Tf_PyObjectFinderBase const **x = _finders->Find(type))
        finder = *x;
    if (finder)
        return finder->Find(objPtr);
    return object();
}
    

Tf_PyObjectFinderBase::~Tf_PyObjectFinderBase() {}

PXR_NAMESPACE_CLOSE_SCOPE
