//
// Copyright 2023 Pixar
//
// Licensed under the Apache License, Version 2.0 (the "Apache License")
// with the following modification; you may not use this file except in
// compliance with the Apache License and the following modification to it:
// Section 6. Trademarks. is deleted and replaced with:
//
// 6. Trademarks. This License does not grant permission to use the trade
//    names, trademarks, service marks, or product names of the Licensor
//    and its affiliates, except as required to comply with Section 4(c) of
//    the License and to reproduce the content of the NOTICE file.
//
// You may obtain a copy of the Apache License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the Apache License with the above modification is
// distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied. See the Apache License for the specific
// language governing permissions and limitations under the Apache License.
//
////////////////////////////////////////////////////////////////////////

/* ************************************************************************** */
/* ** This file is generated by a script.  Do not edit directly.  Edit     ** */
/* ** defs.py or the (*)Schema.template.cpp files to make changes.         ** */
/* ************************************************************************** */

#include "pxr/imaging/hd/basisCurvesSchema.h"
#include "pxr/imaging/hd/retainedDataSource.h"

#include "pxr/base/trace/trace.h"


PXR_NAMESPACE_OPEN_SCOPE

TF_DEFINE_PUBLIC_TOKENS(HdBasisCurvesSchemaTokens,
    HDBASISCURVES_SCHEMA_TOKENS);



HdBasisCurvesTopologySchema
HdBasisCurvesSchema::GetTopology()
{
    return HdBasisCurvesTopologySchema(_GetTypedDataSource<HdContainerDataSource>(
        HdBasisCurvesSchemaTokens->topology));
}

HdGeomSubsetsSchema
HdBasisCurvesSchema::GetGeomSubsets()
{
    return HdGeomSubsetsSchema(_GetTypedDataSource<HdContainerDataSource>(
        HdBasisCurvesSchemaTokens->geomSubsets));
}

/*static*/
HdContainerDataSourceHandle
HdBasisCurvesSchema::BuildRetained(
        const HdContainerDataSourceHandle &topology,
        const HdContainerDataSourceHandle &geomSubsets
)
{
    TfToken names[2];
    HdDataSourceBaseHandle values[2];

    size_t count = 0;
    if (topology) {
        names[count] = HdBasisCurvesSchemaTokens->topology;
        values[count++] = topology;
    }

    if (geomSubsets) {
        names[count] = HdBasisCurvesSchemaTokens->geomSubsets;
        values[count++] = geomSubsets;
    }

    return HdRetainedContainerDataSource::New(count, names, values);
}

/*static*/
HdBasisCurvesSchema
HdBasisCurvesSchema::GetFromParent(
        const HdContainerDataSourceHandle &fromParentContainer)
{
    return HdBasisCurvesSchema(
        fromParentContainer
        ? HdContainerDataSource::Cast(fromParentContainer->Get(
                HdBasisCurvesSchemaTokens->basisCurves))
        : nullptr);
}

/*static*/
const TfToken &
HdBasisCurvesSchema::GetSchemaToken()
{
    return HdBasisCurvesSchemaTokens->basisCurves;
} 
/*static*/
const HdDataSourceLocator &
HdBasisCurvesSchema::GetDefaultLocator()
{
    static const HdDataSourceLocator locator(
        HdBasisCurvesSchemaTokens->basisCurves
    );
    return locator;
} 
/*static*/
const HdDataSourceLocator &
HdBasisCurvesSchema::GetTopologyLocator()
{
    static const HdDataSourceLocator locator(
        HdBasisCurvesSchemaTokens->basisCurves,
        HdBasisCurvesSchemaTokens->topology
    );
    return locator;
}

/*static*/
const HdDataSourceLocator &
HdBasisCurvesSchema::GetGeomSubsetsLocator()
{
    static const HdDataSourceLocator locator(
        HdBasisCurvesSchemaTokens->basisCurves,
        HdBasisCurvesSchemaTokens->geomSubsets
    );
    return locator;
}


HdBasisCurvesSchema::Builder &
HdBasisCurvesSchema::Builder::SetTopology(
    const HdContainerDataSourceHandle &topology)
{
    _topology = topology;
    return *this;
}

HdBasisCurvesSchema::Builder &
HdBasisCurvesSchema::Builder::SetGeomSubsets(
    const HdContainerDataSourceHandle &geomSubsets)
{
    _geomSubsets = geomSubsets;
    return *this;
}

HdContainerDataSourceHandle
HdBasisCurvesSchema::Builder::Build()
{
    return HdBasisCurvesSchema::BuildRetained(
        _topology,
        _geomSubsets
    );
}


PXR_NAMESPACE_CLOSE_SCOPE