//
// Copyright 2023 Pixar
//
// Licensed under the Apache License, Version 2.0 (the "Apache License")
// with the following modification; you may not use this file except in
// compliance with the Apache License and the following modification to it:
// Section 6. Trademarks. is deleted and replaced with:
//
// 6. Trademarks. This License does not grant permission to use the trade
//    names, trademarks, service marks, or product names of the Licensor
//    and its affiliates, except as required to comply with Section 4(c) of
//    the License and to reproduce the content of the NOTICE file.
//
// You may obtain a copy of the Apache License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the Apache License with the above modification is
// distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied. See the Apache License for the specific
// language governing permissions and limitations under the Apache License.
//
////////////////////////////////////////////////////////////////////////

/* ************************************************************************** */
/* ** This file is generated by a script.  Do not edit directly.  Edit     ** */
/* ** defs.py or the (*)Schema.template.cpp files to make changes.         ** */
/* ************************************************************************** */

#include "pxr/imaging/hd/lensDistortionSchema.h"
#include "pxr/imaging/hd/retainedDataSource.h"
#include "pxr/imaging/hd/cameraSchema.h"

#include "pxr/base/trace/trace.h"


PXR_NAMESPACE_OPEN_SCOPE

TF_DEFINE_PUBLIC_TOKENS(HdLensDistortionSchemaTokens,
    HDLENSDISTORTION_SCHEMA_TOKENS);



HdTokenDataSourceHandle
HdLensDistortionSchema::GetType()
{
    return _GetTypedDataSource<HdTokenDataSource>(
        HdLensDistortionSchemaTokens->type);
}

HdFloatDataSourceHandle
HdLensDistortionSchema::GetK1()
{
    return _GetTypedDataSource<HdFloatDataSource>(
        HdLensDistortionSchemaTokens->k1);
}

HdFloatDataSourceHandle
HdLensDistortionSchema::GetK2()
{
    return _GetTypedDataSource<HdFloatDataSource>(
        HdLensDistortionSchemaTokens->k2);
}

HdVec2fDataSourceHandle
HdLensDistortionSchema::GetCenter()
{
    return _GetTypedDataSource<HdVec2fDataSource>(
        HdLensDistortionSchemaTokens->center);
}

HdFloatDataSourceHandle
HdLensDistortionSchema::GetAnaSq()
{
    return _GetTypedDataSource<HdFloatDataSource>(
        HdLensDistortionSchemaTokens->anaSq);
}

HdVec2fDataSourceHandle
HdLensDistortionSchema::GetAsym()
{
    return _GetTypedDataSource<HdVec2fDataSource>(
        HdLensDistortionSchemaTokens->asym);
}

HdFloatDataSourceHandle
HdLensDistortionSchema::GetScale()
{
    return _GetTypedDataSource<HdFloatDataSource>(
        HdLensDistortionSchemaTokens->scale);
}

HdFloatDataSourceHandle
HdLensDistortionSchema::GetIor()
{
    return _GetTypedDataSource<HdFloatDataSource>(
        HdLensDistortionSchemaTokens->ior);
}

/*static*/
HdContainerDataSourceHandle
HdLensDistortionSchema::BuildRetained(
        const HdTokenDataSourceHandle &type,
        const HdFloatDataSourceHandle &k1,
        const HdFloatDataSourceHandle &k2,
        const HdVec2fDataSourceHandle &center,
        const HdFloatDataSourceHandle &anaSq,
        const HdVec2fDataSourceHandle &asym,
        const HdFloatDataSourceHandle &scale,
        const HdFloatDataSourceHandle &ior
)
{
    TfToken names[8];
    HdDataSourceBaseHandle values[8];

    size_t count = 0;
    if (type) {
        names[count] = HdLensDistortionSchemaTokens->type;
        values[count++] = type;
    }

    if (k1) {
        names[count] = HdLensDistortionSchemaTokens->k1;
        values[count++] = k1;
    }

    if (k2) {
        names[count] = HdLensDistortionSchemaTokens->k2;
        values[count++] = k2;
    }

    if (center) {
        names[count] = HdLensDistortionSchemaTokens->center;
        values[count++] = center;
    }

    if (anaSq) {
        names[count] = HdLensDistortionSchemaTokens->anaSq;
        values[count++] = anaSq;
    }

    if (asym) {
        names[count] = HdLensDistortionSchemaTokens->asym;
        values[count++] = asym;
    }

    if (scale) {
        names[count] = HdLensDistortionSchemaTokens->scale;
        values[count++] = scale;
    }

    if (ior) {
        names[count] = HdLensDistortionSchemaTokens->ior;
        values[count++] = ior;
    }

    return HdRetainedContainerDataSource::New(count, names, values);
}

/*static*/
HdLensDistortionSchema
HdLensDistortionSchema::GetFromParent(
        const HdContainerDataSourceHandle &fromParentContainer)
{
    return HdLensDistortionSchema(
        fromParentContainer
        ? HdContainerDataSource::Cast(fromParentContainer->Get(
                HdLensDistortionSchemaTokens->lensDistortion))
        : nullptr);
}

/*static*/
const TfToken &
HdLensDistortionSchema::GetSchemaToken()
{
    return HdLensDistortionSchemaTokens->lensDistortion;
}

/*static*/
const HdDataSourceLocator &
HdLensDistortionSchema::GetDefaultLocator()
{
    static const HdDataSourceLocator locator(
        HdCameraSchemaTokens->camera,
        HdLensDistortionSchemaTokens->lensDistortion
    );
    return locator;
} 

/*static*/
HdTokenDataSourceHandle
HdLensDistortionSchema::BuildTypeDataSource(
    const TfToken &type)
{
    if (type == HdLensDistortionSchemaTokens->standard) {
        static const HdRetainedTypedSampledDataSource<TfToken>::Handle ds =
            HdRetainedTypedSampledDataSource<TfToken>::New(type);
        return ds;
    }
    if (type == HdLensDistortionSchemaTokens->fisheye) {
        static const HdRetainedTypedSampledDataSource<TfToken>::Handle ds =
            HdRetainedTypedSampledDataSource<TfToken>::New(type);
        return ds;
    }

    // fallback for unknown token
    return HdRetainedTypedSampledDataSource<TfToken>::New(type);
}

HdLensDistortionSchema::Builder &
HdLensDistortionSchema::Builder::SetType(
    const HdTokenDataSourceHandle &type)
{
    _type = type;
    return *this;
}

HdLensDistortionSchema::Builder &
HdLensDistortionSchema::Builder::SetK1(
    const HdFloatDataSourceHandle &k1)
{
    _k1 = k1;
    return *this;
}

HdLensDistortionSchema::Builder &
HdLensDistortionSchema::Builder::SetK2(
    const HdFloatDataSourceHandle &k2)
{
    _k2 = k2;
    return *this;
}

HdLensDistortionSchema::Builder &
HdLensDistortionSchema::Builder::SetCenter(
    const HdVec2fDataSourceHandle &center)
{
    _center = center;
    return *this;
}

HdLensDistortionSchema::Builder &
HdLensDistortionSchema::Builder::SetAnaSq(
    const HdFloatDataSourceHandle &anaSq)
{
    _anaSq = anaSq;
    return *this;
}

HdLensDistortionSchema::Builder &
HdLensDistortionSchema::Builder::SetAsym(
    const HdVec2fDataSourceHandle &asym)
{
    _asym = asym;
    return *this;
}

HdLensDistortionSchema::Builder &
HdLensDistortionSchema::Builder::SetScale(
    const HdFloatDataSourceHandle &scale)
{
    _scale = scale;
    return *this;
}

HdLensDistortionSchema::Builder &
HdLensDistortionSchema::Builder::SetIor(
    const HdFloatDataSourceHandle &ior)
{
    _ior = ior;
    return *this;
}

HdContainerDataSourceHandle
HdLensDistortionSchema::Builder::Build()
{
    return HdLensDistortionSchema::BuildRetained(
        _type,
        _k1,
        _k2,
        _center,
        _anaSq,
        _asym,
        _scale,
        _ior
    );
}


PXR_NAMESPACE_CLOSE_SCOPE