//
// Copyright 2023 Pixar
//
// Licensed under the Apache License, Version 2.0 (the "Apache License")
// with the following modification; you may not use this file except in
// compliance with the Apache License and the following modification to it:
// Section 6. Trademarks. is deleted and replaced with:
//
// 6. Trademarks. This License does not grant permission to use the trade
//    names, trademarks, service marks, or product names of the Licensor
//    and its affiliates, except as required to comply with Section 4(c) of
//    the License and to reproduce the content of the NOTICE file.
//
// You may obtain a copy of the Apache License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the Apache License with the above modification is
// distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied. See the Apache License for the specific
// language governing permissions and limitations under the Apache License.
//
////////////////////////////////////////////////////////////////////////

/* ************************************************************************** */
/* ** This file is generated by a script.  Do not edit directly.  Edit     ** */
/* ** defs.py or the (*)Schema.template.h files to make changes.           ** */
/* ************************************************************************** */

#ifndef PXR_IMAGING_HD_PRIMVARS_SCHEMA_H
#define PXR_IMAGING_HD_PRIMVARS_SCHEMA_H

#include "pxr/imaging/hd/api.h"

#include "pxr/imaging/hd/primvarSchema.h"


PXR_NAMESPACE_OPEN_SCOPE

//-----------------------------------------------------------------------------

#define HDPRIMVARS_SCHEMA_TOKENS \
    (primvars) \
    (points) \
    (normals) \
    (widths) \

TF_DECLARE_PUBLIC_TOKENS(HdPrimvarsSchemaTokens, HD_API,
    HDPRIMVARS_SCHEMA_TOKENS);

//-----------------------------------------------------------------------------

class HdPrimvarsSchema : public HdSchema
{
public:
    HdPrimvarsSchema(HdContainerDataSourceHandle container)
    : HdSchema(container) {}

    HD_API
    TfTokenVector GetPrimvarNames();

    HD_API
    HdPrimvarSchema GetPrimvar(TfToken name);


    /// Retrieves a container data source with the schema's default name token
    /// "primvars" from the parent container and constructs a
    /// HdPrimvarsSchema instance.
    /// Because the requested container data source may not exist, the result
    /// should be checked with IsDefined() or a bool comparison before use.
    HD_API
    static HdPrimvarsSchema GetFromParent(
        const HdContainerDataSourceHandle &fromParentContainer);

    /// Returns a token where the container representing this schema is found in
    /// a container by default.
    HD_API
    static const TfToken &GetSchemaToken();

    /// Returns an HdDataSourceLocator (relative to the prim-level data source)
    /// where the container representing this schema is found by default.
    HD_API
    static const HdDataSourceLocator &GetDefaultLocator();


    /// Returns an HdDataSourceLocator (relative to the prim-level data source)
    /// where the points data source can be found.
    /// This is often useful for checking intersection against the
    /// HdDataSourceLocatorSet sent with HdDataSourceObserver::PrimsDirtied.
    HD_API
    static const HdDataSourceLocator &GetPointsLocator();

    /// Returns an HdDataSourceLocator (relative to the prim-level data source)
    /// where the normals data source can be found.
    /// This is often useful for checking intersection against the
    /// HdDataSourceLocatorSet sent with HdDataSourceObserver::PrimsDirtied.
    HD_API
    static const HdDataSourceLocator &GetNormalsLocator();

    /// Returns an HdDataSourceLocator (relative to the prim-level data source)
    /// where the widths data source can be found.
    /// This is often useful for checking intersection against the
    /// HdDataSourceLocatorSet sent with HdDataSourceObserver::PrimsDirtied.
    HD_API
    static const HdDataSourceLocator &GetWidthsLocator();

};

PXR_NAMESPACE_CLOSE_SCOPE

#endif