//
// Copyright 2023 Pixar
//
// Licensed under the Apache License, Version 2.0 (the "Apache License")
// with the following modification; you may not use this file except in
// compliance with the Apache License and the following modification to it:
// Section 6. Trademarks. is deleted and replaced with:
//
// 6. Trademarks. This License does not grant permission to use the trade
//    names, trademarks, service marks, or product names of the Licensor
//    and its affiliates, except as required to comply with Section 4(c) of
//    the License and to reproduce the content of the NOTICE file.
//
// You may obtain a copy of the Apache License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the Apache License with the above modification is
// distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied. See the Apache License for the specific
// language governing permissions and limitations under the Apache License.
//
////////////////////////////////////////////////////////////////////////

/* ************************************************************************** */
/* ** This file is generated by a script.  Do not edit directly.  Edit     ** */
/* ** defs.py or the (*)Schema.template.h files to make changes.           ** */
/* ************************************************************************** */

#ifndef PXR_IMAGING_HD_SUBDIVISION_TAGS_SCHEMA_H
#define PXR_IMAGING_HD_SUBDIVISION_TAGS_SCHEMA_H

#include "pxr/imaging/hd/api.h"

#include "pxr/imaging/hd/schema.h" 

PXR_NAMESPACE_OPEN_SCOPE

//-----------------------------------------------------------------------------

#define HDSUBDIVISIONTAGS_SCHEMA_TOKENS \
    (subdivisionTags) \
    (faceVaryingLinearInterpolation) \
    (interpolateBoundary) \
    (triangleSubdivisionRule) \
    (cornerIndices) \
    (cornerSharpnesses) \
    (creaseIndices) \
    (creaseLengths) \
    (creaseSharpnesses) \

TF_DECLARE_PUBLIC_TOKENS(HdSubdivisionTagsSchemaTokens, HD_API,
    HDSUBDIVISIONTAGS_SCHEMA_TOKENS);

//-----------------------------------------------------------------------------

class HdSubdivisionTagsSchema : public HdSchema
{
public:
    HdSubdivisionTagsSchema(HdContainerDataSourceHandle container)
    : HdSchema(container) {}

    //ACCESSORS

    HD_API
    HdTokenDataSourceHandle GetFaceVaryingLinearInterpolation();
    HD_API
    HdTokenDataSourceHandle GetInterpolateBoundary();
    HD_API
    HdTokenDataSourceHandle GetTriangleSubdivisionRule();
    HD_API
    HdIntArrayDataSourceHandle GetCornerIndices();
    HD_API
    HdFloatArrayDataSourceHandle GetCornerSharpnesses();
    HD_API
    HdIntArrayDataSourceHandle GetCreaseIndices();
    HD_API
    HdIntArrayDataSourceHandle GetCreaseLengths();
    HD_API
    HdFloatArrayDataSourceHandle GetCreaseSharpnesses();

    // RETRIEVING AND CONSTRUCTING

    /// Builds a container data source which includes the provided child data
    /// sources. Parameters with nullptr values are excluded. This is a
    /// low-level interface. For cases in which it's desired to define
    /// the container with a sparse set of child fields, the Builder class
    /// is often more convenient and readable.
    HD_API
    static HdContainerDataSourceHandle
    BuildRetained(
        const HdTokenDataSourceHandle &faceVaryingLinearInterpolation,
        const HdTokenDataSourceHandle &interpolateBoundary,
        const HdTokenDataSourceHandle &triangleSubdivisionRule,
        const HdIntArrayDataSourceHandle &cornerIndices,
        const HdFloatArrayDataSourceHandle &cornerSharpnesses,
        const HdIntArrayDataSourceHandle &creaseIndices,
        const HdIntArrayDataSourceHandle &creaseLengths,
        const HdFloatArrayDataSourceHandle &creaseSharpnesses
    );

    /// \class HdSubdivisionTagsSchema::Builder
    /// 
    /// Utility class for setting sparse sets of child data source fields to be
    /// filled as arguments into BuildRetained. Because all setter methods
    /// return a reference to the instance, this can be used in the "builder
    /// pattern" form.
    class Builder
    {
    public:
        HD_API
        Builder &SetFaceVaryingLinearInterpolation(
            const HdTokenDataSourceHandle &faceVaryingLinearInterpolation);
        HD_API
        Builder &SetInterpolateBoundary(
            const HdTokenDataSourceHandle &interpolateBoundary);
        HD_API
        Builder &SetTriangleSubdivisionRule(
            const HdTokenDataSourceHandle &triangleSubdivisionRule);
        HD_API
        Builder &SetCornerIndices(
            const HdIntArrayDataSourceHandle &cornerIndices);
        HD_API
        Builder &SetCornerSharpnesses(
            const HdFloatArrayDataSourceHandle &cornerSharpnesses);
        HD_API
        Builder &SetCreaseIndices(
            const HdIntArrayDataSourceHandle &creaseIndices);
        HD_API
        Builder &SetCreaseLengths(
            const HdIntArrayDataSourceHandle &creaseLengths);
        HD_API
        Builder &SetCreaseSharpnesses(
            const HdFloatArrayDataSourceHandle &creaseSharpnesses);

        /// Returns a container data source containing the members set thus far.
        HD_API
        HdContainerDataSourceHandle Build();

    private:
        HdTokenDataSourceHandle _faceVaryingLinearInterpolation;
        HdTokenDataSourceHandle _interpolateBoundary;
        HdTokenDataSourceHandle _triangleSubdivisionRule;
        HdIntArrayDataSourceHandle _cornerIndices;
        HdFloatArrayDataSourceHandle _cornerSharpnesses;
        HdIntArrayDataSourceHandle _creaseIndices;
        HdIntArrayDataSourceHandle _creaseLengths;
        HdFloatArrayDataSourceHandle _creaseSharpnesses;
    };

    /// Retrieves a container data source with the schema's default name token
    /// "subdivisionTags" from the parent container and constructs a
    /// HdSubdivisionTagsSchema instance.
    /// Because the requested container data source may not exist, the result
    /// should be checked with IsDefined() or a bool comparison before use.
    HD_API
    static HdSubdivisionTagsSchema GetFromParent(
        const HdContainerDataSourceHandle &fromParentContainer);

    /// Returns a token where the container representing this schema is found in
    /// a container by default.
    HD_API
    static const TfToken &GetSchemaToken();

    /// Returns an HdDataSourceLocator (relative to the prim-level data source)
    /// where the container representing this schema is found by default.
    HD_API
    static const HdDataSourceLocator &GetDefaultLocator();

};

PXR_NAMESPACE_CLOSE_SCOPE

#endif