//
// Copyright 2016 Pixar
//
// Licensed under the Apache License, Version 2.0 (the "Apache License")
// with the following modification; you may not use this file except in
// compliance with the Apache License and the following modification to it:
// Section 6. Trademarks. is deleted and replaced with:
//
// 6. Trademarks. This License does not grant permission to use the trade
//    names, trademarks, service marks, or product names of the Licensor
//    and its affiliates, except as required to comply with Section 4(c) of
//    the License and to reproduce the content of the NOTICE file.
//
// You may obtain a copy of the Apache License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the Apache License with the above modification is
// distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied. See the Apache License for the specific
// language governing permissions and limitations under the Apache License.
//
#include "pxr/imaging/hd/tokens.h"

PXR_NAMESPACE_OPEN_SCOPE

// XXX: Remove this in 24.05. Until then, external users may
// set this to true to maintain compatability with hydra backends that have
// not yet been updated to use the new hydra-namespaced names.
TF_DEFINE_ENV_SETTING(HD_USE_DEPRECATED_INSTANCER_PRIMVAR_NAMES, false,
    "enable to use pre-23.11 internal instancer primvar names");

TF_DEFINE_PUBLIC_TOKENS(HdTokens, HD_TOKENS);

TF_DEFINE_PUBLIC_TOKENS(HdInstancerTokens, HD_INSTANCER_TOKENS);

TF_DEFINE_PUBLIC_TOKENS(HdReprTokens, HD_REPR_TOKENS);

TF_DEFINE_PUBLIC_TOKENS(HdCullStyleTokens, HD_CULLSTYLE_TOKENS);

TF_DEFINE_PUBLIC_TOKENS(HdPerfTokens, HD_PERF_TOKENS);

TF_DEFINE_PUBLIC_TOKENS(HdShaderTokens, HD_SHADER_TOKENS);

TF_DEFINE_PUBLIC_TOKENS(HdRenderTagTokens, HD_RENDERTAG_TOKENS);

TF_DEFINE_PUBLIC_TOKENS(HdRenderContextTokens, HD_RENDER_CONTEXT_TOKENS);

TF_DEFINE_PUBLIC_TOKENS(HdMaterialTagTokens, HD_MATERIALTAG_TOKENS);

TF_DEFINE_PUBLIC_TOKENS(HdMaterialTerminalTokens, HD_MATERIAL_TERMINAL_TOKENS);

TF_DEFINE_PUBLIC_TOKENS(HdModelDrawModeTokens, HD_MODEL_DRAWMODE_TOKENS);

TF_DEFINE_PUBLIC_TOKENS(HdOptionTokens, HD_OPTION_TOKENS);

TF_DEFINE_PUBLIC_TOKENS(HdRprimTypeTokens, HD_RPRIMTYPE_TOKENS);
TF_DEFINE_PUBLIC_TOKENS(HdSprimTypeTokens, HD_SPRIMTYPE_TOKENS);
TF_DEFINE_PUBLIC_TOKENS(HdBprimTypeTokens, HD_BPRIMTYPE_TOKENS);

TF_DEFINE_PUBLIC_TOKENS(HdPrimTypeTokens, HD_PRIMTYPE_TOKENS);

TF_DEFINE_PUBLIC_TOKENS(HdPrimvarRoleTokens, HD_PRIMVAR_ROLE_TOKENS);

TF_DEFINE_PUBLIC_TOKENS(HdAovTokens, HD_AOV_TOKENS);

TF_DEFINE_PUBLIC_TOKENS(HdRenderSettingsTokens, HD_RENDER_SETTINGS_TOKENS);

TF_DEFINE_PUBLIC_TOKENS(HdRenderSettingsPrimTokens,
                        HD_RENDER_SETTINGS_PRIM_TOKENS);

TF_DEFINE_PUBLIC_TOKENS(HdAspectRatioConformPolicyTokens, 
                        HD_ASPECT_RATIO_CONFORM_POLICY);

TF_DEFINE_PUBLIC_TOKENS(HdResourceTypeTokens, HD_RESOURCE_TYPE_TOKENS);

TF_DEFINE_PUBLIC_TOKENS(HdSceneIndexEmulationTokens, 
    HD_SCENE_INDEX_EMULATION_TOKENS);

bool HdPrimTypeIsGprim(TfToken const& primType)
{
    return (primType == HdPrimTypeTokens->mesh ||
            primType == HdPrimTypeTokens->basisCurves ||
            primType == HdPrimTypeTokens->points ||
            primType == HdPrimTypeTokens->volume);
}

const TfTokenVector &HdLightPrimTypeTokens()
{
    static const TfTokenVector vec = {
        HdPrimTypeTokens->simpleLight,
        HdPrimTypeTokens->cylinderLight,
        HdPrimTypeTokens->diskLight,
        HdPrimTypeTokens->distantLight,
        HdPrimTypeTokens->domeLight,
        HdPrimTypeTokens->light,
        HdPrimTypeTokens->pluginLight,
        HdPrimTypeTokens->rectLight,
        HdPrimTypeTokens->sphereLight,
        HdPrimTypeTokens->meshLight
    };
    return vec;
}


bool HdPrimTypeIsLight(TfToken const& primType)
{
    for (const TfToken &lightPrimType : HdLightPrimTypeTokens()) {
        if (primType == lightPrimType) {
            return true;
        }
    }
    return false;
}

TfToken HdAovTokensMakePrimvar(TfToken const& primvar)
{
    return TfToken(
        HdAovTokens->primvars.GetString() +
        primvar.GetString());
}

TfToken HdAovTokensMakeLpe(TfToken const& lpe)
{
    return TfToken(
        HdAovTokens->lpe.GetString() +
        lpe.GetString());
}

TfToken HdAovTokensMakeShader(TfToken const& shader)
{
    return TfToken(
        HdAovTokens->shader.GetString() +
        shader.GetString());
}

PXR_NAMESPACE_CLOSE_SCOPE

