//
// Copyright 2023 Pixar
//
// Licensed under the Apache License, Version 2.0 (the "Apache License")
// with the following modification; you may not use this file except in
// compliance with the Apache License and the following modification to it:
// Section 6. Trademarks. is deleted and replaced with:
//
// 6. Trademarks. This License does not grant permission to use the trade
//    names, trademarks, service marks, or product names of the Licensor
//    and its affiliates, except as required to comply with Section 4(c) of
//    the License and to reproduce the content of the NOTICE file.
//
// You may obtain a copy of the Apache License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the Apache License with the above modification is
// distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied. See the Apache License for the specific
// language governing permissions and limitations under the Apache License.
//
////////////////////////////////////////////////////////////////////////

/* ************************************************************************** */
/* ** This file is generated by a script.  Do not edit directly.  Edit     ** */
/* ** defs.py or the (*)Schema.template.cpp files to make changes.         ** */
/* ************************************************************************** */

#include "pxr/imaging/hd/xformSchema.h"
#include "pxr/imaging/hd/retainedDataSource.h"

#include "pxr/base/trace/trace.h"


PXR_NAMESPACE_OPEN_SCOPE

TF_DEFINE_PUBLIC_TOKENS(HdXformSchemaTokens,
    HDXFORM_SCHEMA_TOKENS);



HdMatrixDataSourceHandle
HdXformSchema::GetMatrix()
{
    return _GetTypedDataSource<HdMatrixDataSource>(
        HdXformSchemaTokens->matrix);
}

HdBoolDataSourceHandle
HdXformSchema::GetResetXformStack()
{
    return _GetTypedDataSource<HdBoolDataSource>(
        HdXformSchemaTokens->resetXformStack);
}

/*static*/
HdContainerDataSourceHandle
HdXformSchema::BuildRetained(
        const HdMatrixDataSourceHandle &matrix,
        const HdBoolDataSourceHandle &resetXformStack
)
{
    TfToken names[2];
    HdDataSourceBaseHandle values[2];

    size_t count = 0;
    if (matrix) {
        names[count] = HdXformSchemaTokens->matrix;
        values[count++] = matrix;
    }

    if (resetXformStack) {
        names[count] = HdXformSchemaTokens->resetXformStack;
        values[count++] = resetXformStack;
    }

    return HdRetainedContainerDataSource::New(count, names, values);
}

/*static*/
HdXformSchema
HdXformSchema::GetFromParent(
        const HdContainerDataSourceHandle &fromParentContainer)
{
    return HdXformSchema(
        fromParentContainer
        ? HdContainerDataSource::Cast(fromParentContainer->Get(
                HdXformSchemaTokens->xform))
        : nullptr);
}

/*static*/
const TfToken &
HdXformSchema::GetSchemaToken()
{
    return HdXformSchemaTokens->xform;
} 
/*static*/
const HdDataSourceLocator &
HdXformSchema::GetDefaultLocator()
{
    static const HdDataSourceLocator locator(
        HdXformSchemaTokens->xform
    );
    return locator;
} 
HdXformSchema::Builder &
HdXformSchema::Builder::SetMatrix(
    const HdMatrixDataSourceHandle &matrix)
{
    _matrix = matrix;
    return *this;
}

HdXformSchema::Builder &
HdXformSchema::Builder::SetResetXformStack(
    const HdBoolDataSourceHandle &resetXformStack)
{
    _resetXformStack = resetXformStack;
    return *this;
}

HdContainerDataSourceHandle
HdXformSchema::Builder::Build()
{
    return HdXformSchema::BuildRetained(
        _matrix,
        _resetXformStack
    );
}


PXR_NAMESPACE_CLOSE_SCOPE