//
// Copyright 2023 Pixar
//
// Licensed under the Apache License, Version 2.0 (the "Apache License")
// with the following modification; you may not use this file except in
// compliance with the Apache License and the following modification to it:
// Section 6. Trademarks. is deleted and replaced with:
//
// 6. Trademarks. This License does not grant permission to use the trade
//    names, trademarks, service marks, or product names of the Licensor
//    and its affiliates, except as required to comply with Section 4(c) of
//    the License and to reproduce the content of the NOTICE file.
//
// You may obtain a copy of the Apache License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the Apache License with the above modification is
// distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied. See the Apache License for the specific
// language governing permissions and limitations under the Apache License.
//
////////////////////////////////////////////////////////////////////////

/* ************************************************************************** */
/* ** This file is generated by a script.  Do not edit directly.  Edit     ** */
/* ** defs.py or the (*)Schema.template.cpp files to make changes.         ** */
/* ************************************************************************** */

#include "pxr/usdImaging/usdImaging/usdRenderProductSchema.h"
#include "pxr/imaging/hd/retainedDataSource.h"

#include "pxr/base/trace/trace.h"


PXR_NAMESPACE_OPEN_SCOPE

TF_DEFINE_PUBLIC_TOKENS(UsdImagingUsdRenderProductSchemaTokens,
    USDIMAGINGUSDRENDERPRODUCT_SCHEMA_TOKENS);



HdVec2iDataSourceHandle
UsdImagingUsdRenderProductSchema::GetResolution()
{
    return _GetTypedDataSource<HdVec2iDataSource>(
        UsdImagingUsdRenderProductSchemaTokens->resolution);
}

HdFloatDataSourceHandle
UsdImagingUsdRenderProductSchema::GetPixelAspectRatio()
{
    return _GetTypedDataSource<HdFloatDataSource>(
        UsdImagingUsdRenderProductSchemaTokens->pixelAspectRatio);
}

HdTokenDataSourceHandle
UsdImagingUsdRenderProductSchema::GetAspectRatioConformPolicy()
{
    return _GetTypedDataSource<HdTokenDataSource>(
        UsdImagingUsdRenderProductSchemaTokens->aspectRatioConformPolicy);
}

HdVec4fDataSourceHandle
UsdImagingUsdRenderProductSchema::GetDataWindowNDC()
{
    return _GetTypedDataSource<HdVec4fDataSource>(
        UsdImagingUsdRenderProductSchemaTokens->dataWindowNDC);
}

HdBoolDataSourceHandle
UsdImagingUsdRenderProductSchema::GetDisableMotionBlur()
{
    return _GetTypedDataSource<HdBoolDataSource>(
        UsdImagingUsdRenderProductSchemaTokens->disableMotionBlur);
}

HdPathDataSourceHandle
UsdImagingUsdRenderProductSchema::GetCamera()
{
    return _GetTypedDataSource<HdPathDataSource>(
        UsdImagingUsdRenderProductSchemaTokens->camera);
}

HdTokenDataSourceHandle
UsdImagingUsdRenderProductSchema::GetProductType()
{
    return _GetTypedDataSource<HdTokenDataSource>(
        UsdImagingUsdRenderProductSchemaTokens->productType);
}

HdTokenDataSourceHandle
UsdImagingUsdRenderProductSchema::GetProductName()
{
    return _GetTypedDataSource<HdTokenDataSource>(
        UsdImagingUsdRenderProductSchemaTokens->productName);
}

HdPathArrayDataSourceHandle
UsdImagingUsdRenderProductSchema::GetOrderedVars()
{
    return _GetTypedDataSource<HdPathArrayDataSource>(
        UsdImagingUsdRenderProductSchemaTokens->orderedVars);
}

HdContainerDataSourceHandle
UsdImagingUsdRenderProductSchema::GetNamespacedSettings()
{
    return _GetTypedDataSource<HdContainerDataSource>(
        UsdImagingUsdRenderProductSchemaTokens->namespacedSettings);
}

/*static*/
HdContainerDataSourceHandle
UsdImagingUsdRenderProductSchema::BuildRetained(
        const HdVec2iDataSourceHandle &resolution,
        const HdFloatDataSourceHandle &pixelAspectRatio,
        const HdTokenDataSourceHandle &aspectRatioConformPolicy,
        const HdVec4fDataSourceHandle &dataWindowNDC,
        const HdBoolDataSourceHandle &disableMotionBlur,
        const HdPathDataSourceHandle &camera,
        const HdTokenDataSourceHandle &productType,
        const HdTokenDataSourceHandle &productName,
        const HdPathArrayDataSourceHandle &orderedVars,
        const HdContainerDataSourceHandle &namespacedSettings
)
{
    TfToken names[10];
    HdDataSourceBaseHandle values[10];

    size_t count = 0;
    if (resolution) {
        names[count] = UsdImagingUsdRenderProductSchemaTokens->resolution;
        values[count++] = resolution;
    }

    if (pixelAspectRatio) {
        names[count] = UsdImagingUsdRenderProductSchemaTokens->pixelAspectRatio;
        values[count++] = pixelAspectRatio;
    }

    if (aspectRatioConformPolicy) {
        names[count] = UsdImagingUsdRenderProductSchemaTokens->aspectRatioConformPolicy;
        values[count++] = aspectRatioConformPolicy;
    }

    if (dataWindowNDC) {
        names[count] = UsdImagingUsdRenderProductSchemaTokens->dataWindowNDC;
        values[count++] = dataWindowNDC;
    }

    if (disableMotionBlur) {
        names[count] = UsdImagingUsdRenderProductSchemaTokens->disableMotionBlur;
        values[count++] = disableMotionBlur;
    }

    if (camera) {
        names[count] = UsdImagingUsdRenderProductSchemaTokens->camera;
        values[count++] = camera;
    }

    if (productType) {
        names[count] = UsdImagingUsdRenderProductSchemaTokens->productType;
        values[count++] = productType;
    }

    if (productName) {
        names[count] = UsdImagingUsdRenderProductSchemaTokens->productName;
        values[count++] = productName;
    }

    if (orderedVars) {
        names[count] = UsdImagingUsdRenderProductSchemaTokens->orderedVars;
        values[count++] = orderedVars;
    }

    if (namespacedSettings) {
        names[count] = UsdImagingUsdRenderProductSchemaTokens->namespacedSettings;
        values[count++] = namespacedSettings;
    }

    return HdRetainedContainerDataSource::New(count, names, values);
}

/*static*/
UsdImagingUsdRenderProductSchema
UsdImagingUsdRenderProductSchema::GetFromParent(
        const HdContainerDataSourceHandle &fromParentContainer)
{
    return UsdImagingUsdRenderProductSchema(
        fromParentContainer
        ? HdContainerDataSource::Cast(fromParentContainer->Get(
                UsdImagingUsdRenderProductSchemaTokens->__usdRenderProduct))
        : nullptr);
}

/*static*/
const TfToken &
UsdImagingUsdRenderProductSchema::GetSchemaToken()
{
    return UsdImagingUsdRenderProductSchemaTokens->__usdRenderProduct;
} 
/*static*/
const HdDataSourceLocator &
UsdImagingUsdRenderProductSchema::GetDefaultLocator()
{
    static const HdDataSourceLocator locator(
        UsdImagingUsdRenderProductSchemaTokens->__usdRenderProduct
    );
    return locator;
} 
/*static*/
const HdDataSourceLocator &
UsdImagingUsdRenderProductSchema::GetNamespacedSettingsLocator()
{
    static const HdDataSourceLocator locator(
        UsdImagingUsdRenderProductSchemaTokens->__usdRenderProduct,
        UsdImagingUsdRenderProductSchemaTokens->namespacedSettings
    );
    return locator;
}


UsdImagingUsdRenderProductSchema::Builder &
UsdImagingUsdRenderProductSchema::Builder::SetResolution(
    const HdVec2iDataSourceHandle &resolution)
{
    _resolution = resolution;
    return *this;
}

UsdImagingUsdRenderProductSchema::Builder &
UsdImagingUsdRenderProductSchema::Builder::SetPixelAspectRatio(
    const HdFloatDataSourceHandle &pixelAspectRatio)
{
    _pixelAspectRatio = pixelAspectRatio;
    return *this;
}

UsdImagingUsdRenderProductSchema::Builder &
UsdImagingUsdRenderProductSchema::Builder::SetAspectRatioConformPolicy(
    const HdTokenDataSourceHandle &aspectRatioConformPolicy)
{
    _aspectRatioConformPolicy = aspectRatioConformPolicy;
    return *this;
}

UsdImagingUsdRenderProductSchema::Builder &
UsdImagingUsdRenderProductSchema::Builder::SetDataWindowNDC(
    const HdVec4fDataSourceHandle &dataWindowNDC)
{
    _dataWindowNDC = dataWindowNDC;
    return *this;
}

UsdImagingUsdRenderProductSchema::Builder &
UsdImagingUsdRenderProductSchema::Builder::SetDisableMotionBlur(
    const HdBoolDataSourceHandle &disableMotionBlur)
{
    _disableMotionBlur = disableMotionBlur;
    return *this;
}

UsdImagingUsdRenderProductSchema::Builder &
UsdImagingUsdRenderProductSchema::Builder::SetCamera(
    const HdPathDataSourceHandle &camera)
{
    _camera = camera;
    return *this;
}

UsdImagingUsdRenderProductSchema::Builder &
UsdImagingUsdRenderProductSchema::Builder::SetProductType(
    const HdTokenDataSourceHandle &productType)
{
    _productType = productType;
    return *this;
}

UsdImagingUsdRenderProductSchema::Builder &
UsdImagingUsdRenderProductSchema::Builder::SetProductName(
    const HdTokenDataSourceHandle &productName)
{
    _productName = productName;
    return *this;
}

UsdImagingUsdRenderProductSchema::Builder &
UsdImagingUsdRenderProductSchema::Builder::SetOrderedVars(
    const HdPathArrayDataSourceHandle &orderedVars)
{
    _orderedVars = orderedVars;
    return *this;
}

UsdImagingUsdRenderProductSchema::Builder &
UsdImagingUsdRenderProductSchema::Builder::SetNamespacedSettings(
    const HdContainerDataSourceHandle &namespacedSettings)
{
    _namespacedSettings = namespacedSettings;
    return *this;
}

HdContainerDataSourceHandle
UsdImagingUsdRenderProductSchema::Builder::Build()
{
    return UsdImagingUsdRenderProductSchema::BuildRetained(
        _resolution,
        _pixelAspectRatio,
        _aspectRatioConformPolicy,
        _dataWindowNDC,
        _disableMotionBlur,
        _camera,
        _productType,
        _productName,
        _orderedVars,
        _namespacedSettings
    );
}


PXR_NAMESPACE_CLOSE_SCOPE